Package: aws.signature
Type: Package
Title: Amazon Web Services Request Signatures
Version: 0.6.0
Date: 2020-06-01
Authors@R: c(person("Thomas J.", "Leeper",
                    role = c("aut"),
                    email = "thosjleeper@gmail.com",
                    comment = c(ORCID = "0000-0003-4097-6326")),
             person("Jonathan", "Stott", email = "jonathan.stott@magairports.com", role = c("cre", "aut")),
             person("Mike", "Kaminsky", email = "kaminsky.michael@gmail.com", role = "ctb")
            )
Description: Generates version 2 and version 4 request signatures for Amazon Web Services ('AWS') <https://aws.amazon.com/> Application Programming Interfaces ('APIs') and provides a mechanism for retrieving credentials from environment variables, 'AWS' credentials files, and 'EC2' instance metadata. For use on 'EC2' instances, users will need to install the suggested package 'aws.ec2metadata' <https://cran.r-project.org/package=aws.ec2metadata>.
License: GPL (>= 2)
Imports: digest, base64enc
Suggests: testthat (>= 2.1.0), aws.ec2metadata (>= 0.1.6)
URL: https://github.com/cloudyr/aws.signature
BugReports: https://github.com/cloudyr/aws.signature/issues
RoxygenNote: 7.1.0
NeedsCompilation: no
Packaged: 2020-06-01 09:51:14 UTC; jon_m
Author: Thomas J. Leeper [aut] (<https://orcid.org/0000-0003-4097-6326>),
  Jonathan Stott [cre, aut],
  Mike Kaminsky [ctb]
Maintainer: Jonathan Stott <jonathan.stott@magairports.com>
Repository: CRAN
Date/Publication: 2020-06-01 10:10:02 UTC
Built: R 4.3.3; ; 2025-01-24 04:02:10 UTC; unix
