Package: aws.signature
Type: Package
Title: Amazon Web Services Request Signatures
Version: 0.6.2
Date: 2020-06-01
Authors@R: c(person("Thomas J.", "Leeper",
                    role = "aut",
                    email = "thosjleeper@gmail.com",
                    comment = c(ORCID = "0000-0003-4097-6326")),
             person("Jonathan", "Stott", email = "jonathan.stott@magairports.com", role = "aut"),
             person("Gonzalo", "Diaz", email = "gonzalomartindiaz22@gmail.com", role = c("cre", "aut"),
                    comment = c(ORCID = "0009-0003-9056-4896")),
             person("Mike", "Kaminsky", email = "kaminsky.michael@gmail.com", role = "ctb"),
             person("Mark", "Douthwaite", email = "mark.douthwaite@peak.ai", role = "ctb"),
             person("Jason", "Gofford", email = "jason.gofford@peak.ai", role = "ctb"),
             person("Luke", "Dyer", email = "luke.dyer@peak.ai", role = "ctb")
            )
Description: Generates version 2 and version 4 request signatures for Amazon Web Services ('AWS') <https://aws.amazon.com/> Application Programming Interfaces ('APIs') and provides a mechanism for retrieving credentials from environment variables, 'AWS' credentials files, and 'EC2' instance metadata. For use on 'EC2' instances, users will need to install the suggested package 'aws.ec2metadata' <https://cran.r-project.org/package=aws.ec2metadata>.
License: GPL (>= 2)
Encoding: UTF-8
Imports: digest, base64enc, jsonlite, curl
Suggests: devtools, testthat (>= 2.1.0), aws.ec2metadata (>= 0.1.6)
URL: https://github.com/cloudyr/aws.signature
BugReports: https://github.com/cloudyr/aws.signature/issues
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-07-18 03:25:52 UTC; gonzalo
Author: Thomas J. Leeper [aut] (ORCID: <https://orcid.org/0000-0003-4097-6326>),
  Jonathan Stott [aut],
  Gonzalo Diaz [cre, aut] (ORCID:
    <https://orcid.org/0009-0003-9056-4896>),
  Mike Kaminsky [ctb],
  Mark Douthwaite [ctb],
  Jason Gofford [ctb],
  Luke Dyer [ctb]
Maintainer: Gonzalo Diaz <gonzalomartindiaz22@gmail.com>
Repository: CRAN
Date/Publication: 2026-07-21 05:40:02 UTC
Built: R 4.5.2; ; 2026-07-21 05:50:41 UTC; unix
