# Generated by roxygen2: do not edit by hand

export(assume_role_with_web_identity)
export(canonical_request)
export(default_credentials_file)
export(locate_credentials)
export(read_credentials)
export(signature_v2_auth)
export(signature_v4)
export(signature_v4_auth)
export(string_to_sign)
export(use_credentials)
importFrom(base64enc,base64encode)
importFrom(digest,digest)
importFrom(digest,hmac)
